package com.chinaitop.depot.intelligent.priceApproval.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chinaitop.depot.intelligent.financialSupervision.model.BusinessFsOutConfirmDiffPriceItem;
import com.chinaitop.depot.intelligent.priceApproval.mapper.*;
import com.chinaitop.depot.intelligent.priceApproval.model.*;
import com.chinaitop.depot.intelligent.priceApproval.service.BusinessFsSaleRevenueService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 财务监管-销售收入上缴 服务实现类
 * </p>
 *
 * @author root
 * @since 2024-07-24
 */
@Service
public class BusinessFsSaleRevenueServiceImpl extends ServiceImpl<BusinessFsSaleRevenueMapper, BusinessFsSaleRevenue> implements BusinessFsSaleRevenueService {
    @Resource
    private BusinessFsSaleRevenueMapper saleRevenueMapper;
    @Resource
    private BusinessFsSaleRevenueInspectionLossMapper saleRevenueInspectionLossMapper;
    @Resource
    private BusinessFsSaleRevenueThreeConsistenceMapper threeConsistenceMapper;
    @Resource
    private BusinessFsSaleRevenueSettleMapper settleMapper;
    @Resource
    private BusinessFsSaleRevenueApprovalPriceMapper approvalPriceMapper;
    @Resource
    private BusinessFsSaleRevenueDiffPriceMapper diffPriceMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveData(BusinessFsSaleRevenue param) {
        // 保存主表
        saleRevenueMapper.insert(param);
        // 保存子表-质检损益信息
        this.saveInspectionLoss(param);
        // 保存子表-三账一致性-会计账
        this.saveThreeConsistence(param);
        // 保存子表-三账一致性-实际出库情况
        this.saveOutStorage(param);
        // 保存子表-结算信息
        this.saveSettle(param);
        // 保存子表-原核定入库价格
        this.saveApprovalPrice(param);
        // 保存子表-差价上缴
        this.saveDiffPrice(param);
    }

    @Override
    public List<BusinessContract> getContractInfo(Integer orgId) {
        return saleRevenueMapper.getContractInfo(orgId);
    }

    @Override
    public List<BusinessContractDetail> getContractInfoList(Integer id) {
        return saleRevenueMapper.getContractInfoList(id);
    }

    @Override
    public List<BusinessFsSaleRevenueInspectionLoss> getQualityInfo(Integer orgId, Integer basicStorehouseId, Integer warehouseId) {
        Map<String, Object> map = new HashMap<>();
        map.put("orgId", orgId);
        map.put("basicStorehouseId", basicStorehouseId);
        map.put("warehouseId", warehouseId);
        return saleRevenueMapper.getQualityInfo(map);
    }

    @Override
    public List<BusinessFsSaleRevenueThreeConsistence> getConsistenceAccountInfo(String contractNumber) {
        return saleRevenueMapper.getConsistenceAccountInfo(contractNumber);
    }

    @Override
    public List<BusinessFsSaleRevenueThreeConsistence> getOutStorageInfo(String contractNumber) {
        return saleRevenueMapper.getOutStorageInfo(contractNumber);
    }

    @Override
    public List<DataCPZ> getOutStorageDetailed(String contractNumber) {
        return saleRevenueMapper.getOutStorageDetailed(contractNumber);
    }

    @Override
    public List<BusinessFsSaleRevenueSettle> getSettlementInfo(String contractNumber) {
        return saleRevenueMapper.getSettlementInfo(contractNumber);
    }

    @Override
    public List<BusinessFsSaleRevenueDiffPrice> getDiffPriceInfo(String contractNumber) {
        return saleRevenueMapper.getDiffPriceInfo(contractNumber);
    }

    @Override
    public BusinessFsSaleRevenueApprovalPrice getCardInfo(Integer orgId, Integer basicStorehouseId, Integer warehouseId) {
        Map<String, Object> map = new HashMap<>();
        map.put("orgId", orgId);
        map.put("basicStorehouseId", basicStorehouseId);
        map.put("warehouseId", warehouseId);
        return saleRevenueMapper.getCardInfo(map);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateData(BusinessFsSaleRevenue param) {

    }

    @Override
    public BusinessFsSaleRevenue detailsData(Integer id) {
        // 查询主表
        BusinessFsSaleRevenue saleRevenue = saleRevenueMapper.selectById(id);
        // 查询子表-质检损益信息
        List<BusinessContractDetail> contractInfoList = this.getContractInfoList(saleRevenue.getContractId());
        if (CollectionUtils.isNotEmpty(contractInfoList)) {
            saleRevenue.setContractDetails(contractInfoList);
        }
        // 查询子表-三账一致性-会计账
        // 查询子表-三账一致性-实际出库情况
        // 查询子表-结算信息
        // 查询子表-原核定入库价格
        // 查询子表-差价上缴

        return null;
    }

    @Override
    public Page<BusinessFsSaleRevenue> selectPageList(BusinessFsSaleRevenuePage param) {
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteData(Integer id) {

    }

    // 保存子表-质检损益信息
    private void saveInspectionLoss(BusinessFsSaleRevenue param) {
        if (CollectionUtils.isNotEmpty(param.getInspectionLossList())) {
            param.getInspectionLossList().forEach(inspectionLoss -> {
                inspectionLoss.setFsSaleRevenueId(param.getId());
                saleRevenueInspectionLossMapper.insert(inspectionLoss);
            });
        }
    }

    // 保存子表-三账一致性-会计账
    private void saveThreeConsistence(BusinessFsSaleRevenue param) {
        if (CollectionUtils.isNotEmpty(param.getThreeConsistenceList())) {
            param.getThreeConsistenceList().forEach(threeConsistence -> {
                threeConsistence.setFsSaleRevenueId(param.getId());
                threeConsistence.setProvinceAccountingQuantity(threeConsistence.getAccountingQuantity() == null ? new BigDecimal(0) : threeConsistence.getAccountingQuantity());
                threeConsistence.setAccountingUnitPrice(new BigDecimal(0));
                threeConsistence.setProvinceAccountingUnitPrice(new BigDecimal(0));
                threeConsistence.setProvinceAccountingAmount(threeConsistence.getAccountingAmount() == null ? new BigDecimal(0) : threeConsistence.getAccountingAmount());
                threeConsistenceMapper.insert(threeConsistence);
            });
        }
    }

    // 保存子表-三账一致性-实际出库情况
    private void saveOutStorage(BusinessFsSaleRevenue param) {
        if (CollectionUtils.isNotEmpty(param.getThreeConsistenceList())) {
            param.getThreeConsistenceList().forEach(threeConsistence -> {
                threeConsistence.setFsSaleRevenueId(param.getId());
                threeConsistence.setProvinceLoss(threeConsistence.getLoss());
                threeConsistence.setProvinceCustodyAccountQuantity(threeConsistence.getCustodyAccountQuantity().toString());
                threeConsistence.setProvinceStatisticalAccountQuantity(threeConsistence.getStatisticalAccountQuantity());
                threeConsistence.setProvinceCompleteTime(threeConsistence.getCompleteTime());
                threeConsistenceMapper.insert(threeConsistence);
            });
        }
    }

    // 保存子表-结算信息
    private void saveSettle(BusinessFsSaleRevenue param) {
        if (CollectionUtils.isNotEmpty(param.getRevenueSettleList())) {
            param.getRevenueSettleList().forEach(settle -> {
                settle.setFsSaleRevenueId(param.getId());
                settleMapper.insert(settle);
            });
        }
    }

    // 保存子表-原核定入库价格
    private void saveApprovalPrice(BusinessFsSaleRevenue param) {
        if (CollectionUtils.isNotEmpty(param.getApprovalPriceList())) {
            param.getApprovalPriceList().forEach(approvalPrice -> {
                approvalPrice.setFsSaleRevenueId(param.getId());
                approvalPriceMapper.insert(approvalPrice);
            });
        }
    }

    // 保存子表-差价上缴
    private void saveDiffPrice(BusinessFsSaleRevenue param) {
        if (CollectionUtils.isNotEmpty(param.getDiffPriceList())) {
            param.getDiffPriceList().forEach(diffPrice -> {
                diffPrice.setFsSaleRevenueId(param.getId());
                diffPrice.setProvinceTotalQuantity(diffPrice.getTotalQuantity());
                diffPrice.setProvinceTotalAmount(diffPrice.getTotalAmount());
                diffPriceMapper.insert(diffPrice);
            });
        }
    }
}
